<div id="chart_container-{{ id }}" class="clearfix">
    <div id="chart-{{ id }}"></div>
    <div id="timeline-{{ id }}"></div>
    <div id="legend_container-{{ id }}">
        <div id="legend-{{ id }}"></div>
    </div>
</div>
